/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Collections.Generic;
using Fractals.Geometry;

namespace Fractals.Data
{
    public class LinearFractalLibrary
    {
        private static readonly Dictionary<string, LinearFractal> fractals;
        
        static LinearFractalLibrary()
        {
            fractals = new Dictionary<string, LinearFractal>
                           {
                               {
                                   "Koh curve", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.0f, 0.0f, -0.5f, 0.0f)
                                               , new Line(-0.5f, 0.0f, 0.0f, 0.5f)
                                               , new Line(0.0f, 0.5f, 0.5f, 0.0f)
                                               , new Line(0.5f, 0.0f, 1.0f, 0.0f)
                                           }))
                                   },
                               {
                                   "Tornado", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.4142f, -1.4142f, 0.0f, 0.0f)
                                               , new Line(0.0f, 0.0f, 0.7071f, -0.7071f)
                                           }))
                                   },
                               {
                                   "Multitornado", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.2f, -1.2f, 0.0f, 0.0f)
                                               , new Line(0.0f, 0.0f, 0.6f, -0.6f)
                                           }))
                                   },
                               {
                                   "Vortex", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.732f, -1.732f, 0.0f, 0.0f)
                                               , new Line(0.0f, 0.0f, 0.866f, -0.866f)
                                           }))
                                   },
                               {
                                   "Bush", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, 0.0f, 0.0f, 1.4f),
                                               new Line(0.0f, 1.4f, 0.0f, 0.0f)
                                           }))
                                   },
                               {
                                   "Bush 2", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, 0.5f, 0.5f, 0.5f),
                                               new Line(0.5f, 0.5f, 0.5f, -0.5f),
                                               new Line(0.5f, -0.5f, -0.5f, 0.5f)
                                           }
                                       ))
                                   },
                               {
                                   "Tree", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, 0.0f, 0.0f, 1.2f),
                                               new Line(0.0f, 1.2f, 0.0f, 0.0f)
                                           }))
                                   },
                               {
                                   "Branch", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, 0, 1, 0.5f),
                                               new Line(1, 0.5f, 0.5f, 0f)
                                           }))
                                   },
                               {
                                   "Square", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, 0.5f, -0.5f, -0.5f),
                                               new Line(-0.5f, -0.5f, 0.5f, -0.5f),
                                               new Line(0.5f, -0.5f, 0.5f, 0.5f),
                                               new Line(0.5f, 0.5f, -0.5f, 0.5f)
                                           }))
                                   },
                               {
                                   "Dracon curve", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.0f, 1.0f, -0.0f, -0.0f),
                                               new Line(1.0f, 1.0f, 0.0f, -0.0f)
                                           }
                                       ))
                                   },
                               {
                                   "Minkovsky curve", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-1.0f, 0.0f, -0.5f, 0.0f),
                                               new Line(-0.5f, 0.0f, -0.5f, 0.5f),
                                               new Line(-0.5f, 0.5f, 0.0f, 0.5f),
                                               new Line(0.0f, 0.5f, 0.0f, 0.0f),
                                               new Line(0.0f, 0.0f, 0.0f, -0.5f),
                                               new Line(0.0f, -0.5f, 0.5f, -0.5f),
                                               new Line(0.5f, -0.5f, 0.5f, 0.0f),
                                               new Line(0.5f, 0.0f, 1.0f, 0.0f)
                                           }
                                       ))
                                   },
                               {
                                   "Piphagor tree", new LinearFractal(
                                   new Line(0.0f, -0.5f, 0.0f, 0.5f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(0.0f, 0.5f, -0.4035f, 1.212f),
                                               new Line(0.0f, 0.5f, 0.2535f, 1.212f)
                                           }
                                       ))
                                   },
                               {
                                   "Square curve", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.5f, -0.5f, -0.5f, 0.5f),
                                               new Line(-0.5f, 0.5f, 0.5f, 0.5f),
                                               new Line(0.5f, 0.5f, 0.5f, -0.5f)
                                           }
                                       ))
                                   },
                               {
                                   "Snowflake", new LinearFractal(
                                   new Line(-1.0f, 0.0f, 1.0f, 0.0f),
                                   new Shape(
                                       new[]
                                           {
                                               new Line(-0.8f, 0.0f, 0.0f, 0.8f),
                                               new Line(0.0f, 0.8f, 0.8f, 0.0f),
                                               new Line(0.8f, 0.0f, 0.0f, -0.8f),
                                               new Line(0.0f, -0.8f, -0.8f, 0.0f)
                                           }
                                       ))
                                   }
                           };
        }

        public static ICollection<string> Keys
        {
            get { return fractals.Keys; }
        }

        public static LinearFractal Get(string fractalName)
        {
            return (LinearFractal)fractals[fractalName].Clone();
        }
    }
}
