/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Drawing; 
using Fractals.Geometry;

namespace Fractals.Drawing
{
	public abstract class AbstractDrawer
	{
	    protected AbstractDrawer()
        {
            IsVisible = true;
        }

        public bool IsVisible { get; set; }

	    public void Draw(Graphics g, PositionedObject obj, Viewport viewport)
	    {
            if (IsVisible && IsObjectVisibleInViewport(obj, viewport))
	            DoDraw(g, obj, viewport);            
	    }
        
        protected virtual bool IsObjectVisibleInViewport(PositionedObject obj, Viewport viewport)
        {
            new Region(viewport.CircumscribedRectangle).Intersect(obj.CircumscribedRectangle);
            return new Region(obj.CircumscribedRectangle).IsVisible(viewport.CircumscribedRectangle);
        }

        protected abstract void DoDraw(Graphics g, PositionedObject obj, Viewport viewport);
	}
}
