/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Drawing;
using System.Drawing.Drawing2D;
using Fractals.Geometry;

namespace Fractals.Drawing
{
    public class CoordinateDrawer : AbstractDrawer
    {
        public CoordinateDrawer()
        {
            AxisPen = new Pen(Color.FromArgb(0xAA, 0xAA, 0xFF)) { DashStyle = DashStyle.Dash };
            DoesDrawAxis = true;
            PointPen = new Pen(Color.Black);
            PointSize = 1;
            DoesDrawPoint = false;
            DoesDrawHint = true;
            HintOffset = new PointF(10, 10);
            HintTextPen = new Pen(new SolidBrush(Color.Black)) { Brush = new SolidBrush(Color.Black)};
			HintFont = new Font(FontFamily.GenericSansSerif, 7); 
        }

        public Pen AxisPen { get; set; }
        public bool DoesDrawAxis { get; set; }
        public Pen PointPen { get; set; }
        public float PointSize { get; set; }
        public bool DoesDrawPoint { get; set; }
        public bool DoesDrawHint { get; set; }
        public PointF HintOffset { get; set; }
		public Pen HintTextPen { get; set; }
		public Font HintFont { get; set; }
        
        protected override void DoDraw(Graphics g, PositionedObject obj, Viewport viewport)
        {
			if (DoesDrawAxis)
				DrawAxis(g, obj, viewport);

			if (DoesDrawPoint)
				DrawPoint(g, obj, viewport);

			if (DoesDrawHint)
				DrawHint(g, obj);
        }

        private void DrawHint(Graphics g, PositionedObject obj)
        {
			g.DrawString(string.Format("({0},{1})",
				(int)obj.Center.X, (int)obj.Center.Y), HintFont, HintTextPen.Brush, 
				new PointF(obj.Center.X + HintOffset.X, obj.Center.Y + HintOffset.Y)); 
        }

        private void DrawPoint(Graphics g, PositionedObject obj, Viewport viewport)
        {
        }

        private void DrawAxis(Graphics g, PositionedObject obj, Viewport viewport)
        {
            g.DrawLine(AxisPen, obj.Center.X, viewport.CircumscribedRectangle.Top, obj.Center.X, viewport.CircumscribedRectangle.Bottom);
            g.DrawLine(AxisPen, viewport.CircumscribedRectangle.Left, obj.Center.Y, viewport.CircumscribedRectangle.Right, obj.Center.Y);
        }
    }
}
