/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Drawing;
using System.Drawing.Drawing2D;
using Fractals.Geometry; 

namespace Fractals.Drawing
{
	public class CoordinateSurfaceDrawer : AbstractDrawer 
	{
		public CoordinateSurfaceDrawer()
		{
            GridPen = new Pen(Color.FromArgb(0xEE, 0xEE, 0xFA)) { DashStyle = DashStyle.DashDot };
            GridStep = 10;
            DoesDrawGrid = true;

            AxesPen = new Pen(Color.FromArgb(0xAA, 0xBB, 0xBB));
            AxisBigStep = 10;
            AxisSmallStep = 5;
		    DoesDrawAxis = true;
		}

        public Pen GridPen { get; set; }
        public float GridStep { get; set; }
        public bool DoesDrawGrid { get; set; }
        public Pen AxesPen { get; set; }
        public float AxisBigStep { get; set; }
        public float AxisSmallStep { get; set; }
        public bool DoesDrawAxis { get; set; }
        

        protected override void DoDraw(Graphics g, PositionedObject surface, Viewport viewport)
		{
            if (DoesDrawGrid)
                DrawGrid(g, GridPen, GridStep, surface, viewport);

            if (DoesDrawAxis)
                DrawAxes(g, AxesPen, surface, viewport);
		}

        private static void DrawGrid(Graphics g, Pen pen, float step, PositionedObject surface, Viewport viewport)
        {
            for (var x = surface.Center.X; x < viewport.CircumscribedRectangle.Right; x += step)
                g.DrawLine(pen, x, viewport.CircumscribedRectangle.Bottom, x, viewport.CircumscribedRectangle.Top);

            for (var x = surface.Center.X; x > viewport.CircumscribedRectangle.Left; x -= step)
                g.DrawLine(pen, x, viewport.CircumscribedRectangle.Bottom, x, viewport.CircumscribedRectangle.Top);

            for (var y = surface.Center.Y; y > viewport.CircumscribedRectangle.Top; y -= step)
                g.DrawLine(pen, viewport.CircumscribedRectangle.Left, y, viewport.CircumscribedRectangle.Right, y);

            for (var y = surface.Center.Y; y < viewport.CircumscribedRectangle.Bottom; y += step)
                g.DrawLine(pen, viewport.CircumscribedRectangle.Left, y, viewport.CircumscribedRectangle.Right, y);
        }

        private static void DrawAxes(Graphics g, Pen pen, PositionedObject surface, Viewport viewport)
        {
            g.DrawLine(pen,
                viewport.Center.X - viewport.Size.Width / 2, surface.Center.Y,
                viewport.Center.X + viewport.Size.Width / 2, surface.Center.Y);

            g.DrawLine(pen,
                surface.Center.X, viewport.Center.Y - viewport.Size.Height / 2,
                surface.Center.X, viewport.Center.Y + viewport.Size.Height / 2);
        }
	}
}
