/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Drawing;
using Fractals.Geometry;
using System.Drawing.Drawing2D;

namespace Fractals.Drawing
{
	public class LinearFractalDrawer : AbstractDrawer 
	{
		public LinearFractalDrawer()
		{
			FractalDeep = 5;
			FractalPen = new Pen(Color.Black);
			BaseLinePen = new Pen(Color.Gray) {DashStyle = DashStyle.Dot };
			DoesDrawBaseLines = true;
			DoesDrawDeepestFragmentsOnly = false;
		}

		public int FractalDeep { get; set; }
		public Pen FractalPen { get; set; }
		public Pen BaseLinePen { get; set;}
		public bool DoesDrawBaseLines { get; set; }
		public bool DoesDrawDeepestFragmentsOnly { get; set; }

		protected override void DoDraw(Graphics g, PositionedObject obj, Viewport viewport)
		{
			var fractal = obj as LinearFractal;
            if (fractal == null)
                return;

			FractalFragmentCalculatedHandler drawFragmentHandler = (s, ea) => DrawFragment(g, viewport, ea.BaseLine, ea.Fragment, ea.Deep);
            fractal.FractalFragmentCalculatedEvent += drawFragmentHandler;
		    fractal.Calculate(FractalDeep);
		    fractal.FractalFragmentCalculatedEvent -= drawFragmentHandler; 
		}

		private void DrawFragment(Graphics g, Viewport viewport, Line baseLine, Shape shape, int deep)
		{
            if (DoesDrawDeepestFragmentsOnly && deep != FractalDeep)
				return;
            
            var colorRedStep = FractalPen.Color.R / FractalDeep;
		    var colorGreenStep = FractalPen.Color.G/FractalDeep;
            var colorBlueStep = FractalPen.Color.B / FractalDeep;

            var pen = new Pen(Color.FromArgb(colorRedStep * deep, colorGreenStep * deep, colorBlueStep * deep));
			var shapeDrawer = new ShapeDrawer {LinesPen = pen};
		   
			if (DoesDrawBaseLines)
				g.DrawLine(BaseLinePen, baseLine.Start, baseLine.End);
							
		    shapeDrawer.Draw(g, shape, viewport);
		}						
	}
}
