/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System;
using System.Collections.Generic;
using System.Drawing;
using Fractals.Geometry;

namespace Fractals.Drawing
{
	public class ObjectsCollectionDrawer
	{
        private readonly ObjectDrawerMap drawers;

		public ObjectsCollectionDrawer()
		{
		    drawers = new ObjectDrawerMap(); 
			BackgroundPen = new Pen(Color.White) { Brush = new SolidBrush(Color.White) };  
		}

	    public ObjectDrawerMap Drawers
	    {
            get { return drawers; }
	    }

		public Pen BackgroundPen { get; set; }
        

        public void Draw(Graphics g, Viewport viewport, IEnumerable<PositionedObject> objects)
		{
            TranslateToViewportCenter(g, viewport);
            g.Clip = new Region(viewport.CircumscribedRectangle);
			g.FillRectangle(BackgroundPen.Brush,
				viewport.CircumscribedRectangle.Left, viewport.CircumscribedRectangle.Top,
				viewport.CircumscribedRectangle.Width, viewport.CircumscribedRectangle.Height);

            foreach (var obj in objects)
			{
                var drawer = drawers[obj];
                if  (drawer != null )
                    drawer.Draw(g, obj, viewport);   
			}			
		}

        private static void TranslateToViewportCenter(Graphics g, Viewport viewport)
        {
            g.TranslateTransform(-viewport.Center.X + viewport.Size.Width / 2, -viewport.Center.Y + viewport.Size.Height / 2);
        }
	}
}
