/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System;
using System.Drawing;

namespace Fractals.Geometry
{
	public class Line : ICloneable
	{
	    public Line(PointF start, PointF end)
		{
			Start = start;
			End = end;
		}

		public Line(float startX, float endX, float startY, float endY)
		{
			Start = new PointF(startX, endX);
			End = new PointF(startY, endY); 
		}

	    public PointF Start { get; set; }
	    public PointF End { get; set; }

	    public PointF Vector
	    {
            get { return new PointF(End.X - Start.X, End.Y - Start.Y); }
	    }

	    public PointF Center
	    {
            get { return new PointF((End.X + Start.X) / 2, (End.Y + Start.Y) / 2); }
	    }

	    public float Length
	    {
	        get 
            {
                var vector = Vector;
                return (float)Math.Sqrt(vector.X*vector.X + vector.Y*vector.Y);
            }
	    }

		public void Scale(float scale)
		{
			if (scale == 0)
				return;

			Start = new PointF(Start.X * scale, Start.Y * scale);
			End = new PointF(End.X * scale, End.Y * scale);
		}

		public void Rotate(float angle)
		{
			Start = Start.Rotate(angle);
			End = End.Rotate(angle); 
		}

		public void Move(float dx, float dy)
		{
			Start = new PointF(Start.X + dx, Start.Y + dy);
			End = new PointF(End.X + dx, End.Y + dy);			
		}

		public void Move(PointF dxy)
		{
			Move(dxy.X, dxy.Y);
		}
		
	    public object Clone()
		{
			return new Line(Start, End);
		}	
	}
}
