/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Drawing;
using System;

namespace Fractals.Geometry
{
	public class PositionedObject : ICloneable
	{
	    private PointF center;

		public PositionedObject()
			: this(new PointF(0, 0))
		{ }

		public PositionedObject(PointF center)
		{
			this.center = center;
		}

		public PointF Center 
        { 
            get { return center; }
            set { center = value; }
        }

	    public virtual RectangleF CircumscribedRectangle
	    {
            get { return new RectangleF(float.MinValue, float.MinValue, float.MaxValue, float.MaxValue); }
	    }

		public virtual void MoveTo(float x, float y)
		{
			center.X = x;
			center.Y = y;
		}
		        
        public void Move(float dx, float dy)
        {
			MoveTo(center.X + dx, center.Y + dy); 			
        }

        public void Move(PointF dxy)
        {
            Move(dxy.X, dxy.Y);
        }              

        public void MoveTo(PointF xy)
        {
            MoveTo(xy.X, xy.Y);
        }

		public virtual void Rotate(float angl)
		{
		}

		public virtual void Scale(float scale)
		{
		}

		public virtual object Clone()
		{
			return new PositionedObject(Center);
		}		
	}
}
