/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System;
using System.Collections.Generic;

namespace Fractals.Geometry
{
	public class PositionedObjectsCollection : IEnumerable<PositionedObject> 
	{
		private readonly List<PositionedObject> objects = new List<PositionedObject>();
        private readonly Dictionary<Type, PositionedObject> singleObjects = new Dictionary<Type, PositionedObject>();  

		public void Add(PositionedObject obj)
		{
			objects.Add(obj);  
			if (!singleObjects.ContainsKey(obj.GetType()))
				singleObjects.Add(obj.GetType(), obj); 
		}

		public void Remove(PositionedObject obj)
		{
			objects.Remove(obj);  
            //TODO: add on remove event
		}

		public T Get<T>() where T : PositionedObject 
		{
			return singleObjects.ContainsKey(typeof(T)) ? (T)singleObjects[typeof(T)] : null;
		}

		public IEnumerable<PositionedObject> GetAll() 
		{
			return objects;
		}

        public IEnumerator<PositionedObject> GetEnumerator()
        {
            return objects.GetEnumerator();
        }

		public bool Contains(PositionedObject obj)
		{
			return objects.Contains(obj); 
		}

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            throw new NotImplementedException();
        }
    }
}
