/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using Fractals.Core;
using System.Collections.Generic;

namespace Fractals.Geometry
{
	public class Shape : PositionedObject
	{
		private readonly Line[] lines;

		public Shape(Line[] lines)
		{
			this.lines = lines.DeepClone();
		}

		public Line[] Lines
		{
			get { return lines; }
		}

		public IEnumerable<Line> AbsolutePositionedLines
		{
			get
			{
				foreach (var line in Lines)
					yield return
						new Line(Center.X + line.Start.X, Center.Y + line.Start.Y,
							Center.X + line.End.X, Center.Y + line.End.Y);
			}
		}

		public override void Scale(float scale)
		{
			foreach (var line in lines)
				line.Scale(scale); 			
		}

		public override void Rotate(float angle)
		{
			foreach (var line in lines)
				line.Rotate(angle); 			
		}

		public override object Clone()
		{
			return new Shape(lines);
		}
	}
}
