/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System;
using System.Windows.Forms;
using Fractals.Geometry;
using Fractals.Drawing;
using System.Drawing;  

namespace Fractals.Windows.Controls
{
	public class ViewportCanvas : Panel
	{
		private bool mouseDrag;
		private Point mouseDownPoint;
	    private bool isDrawing;

		public ViewportCanvas()
		{
		    Paint += (sender, args) => DrawObjects();
			MouseMove += canvas_MouseMove;
			MouseDown += canvas_MouseDown;
			Resize += canvas_Resize;
			MouseUp += canvas_MouseUp;
		}	

		public Viewport Viewport { get; set; }
		public PositionedObjectsCollection Objects { get; set; }
		public PositionedObject CursorCoordinate { get; set; }
		public LinearFractal Fractal { get; set; }
        public ObjectsCollectionDrawer ObjectsDrawer { get; set; } 

		public void DrawObjects()
		{
            if (Viewport == null || Objects == null || ObjectsDrawer == null)
				return;

            if (isDrawing)
                return;

		    isDrawing = true;
			CreateGraphics().DoubleBufferedDraw(Width, Height,
                graphics => ObjectsDrawer.Draw(graphics, Viewport, Objects));
            isDrawing = false;
		}

	    private void canvas_MouseMove(object sender, MouseEventArgs e)
		{
            if (Viewport == null || Objects == null || ObjectsDrawer == null)
				return;

			if (mouseDrag)
			{
				if (e.Button == MouseButtons.Left)
				{
					Cursor = Cursors.NoMove2D;
					Viewport.Move(mouseDownPoint.X - e.Location.X, mouseDownPoint.Y - e.Location.Y);					
				}

				if (e.Button == MouseButtons.Right)
				{
					Fractal.Rotate((mouseDownPoint.Y - e.Location.Y)/50.0f);					
				}

				if (e.Button == MouseButtons.Middle)
				{
					var delta = mouseDownPoint.Y - e.Location.Y;
					float scale =  0;
					if (delta > 0)
						scale = 1.05f;
					else if (delta < 0)
						scale = 0.95f;
					Fractal.Scale(scale); 					
				}

				mouseDownPoint = e.Location;			
			}

			CursorCoordinate.MoveTo(Viewport.CircumscribedRectangle.Left + e.Location.X, Viewport.CircumscribedRectangle.Top + e.Location.Y);
			
            if (mouseDrag || ObjectsDrawer.Drawers[CursorCoordinate].IsVisible)
                DrawObjects();
		}

		private void canvas_MouseDown(object sender, MouseEventArgs e)
		{
			mouseDrag = true;
			mouseDownPoint = e.Location;
		}

		private void canvas_MouseUp(object sender, MouseEventArgs e)
		{
			mouseDrag = false;
			Cursor = Cursors.Default;
		}

		private void canvas_Resize(object sender, EventArgs e)
		{
            if (Viewport == null || Objects == null || ObjectsDrawer == null)
				return;
			
			Viewport.Size = Size;
			DrawObjects();			
		}
	}
}
