/*****************************************************************************/
/* Project:     Fractals                                                        
/* Author:      Eugene Sirota                                                   
/* Created:     2008                                                            
/*****************************************************************************/

using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Fractals.Drawing;
using Fractals.Geometry;
using Fractals.Core;
using System;
using Fractals.Data;

namespace Fractals.Windows
{
	public partial class MainForm : Form
	{
	    private readonly PositionedObject coordinateSurface;
		private readonly PositionedObject cursorCoordinate;
		private LinearFractal linearFractal;
	    private readonly ObjectsCollectionDrawer objectsDrawer;
		private readonly LinearFractalDrawer linearFractalDrawer;
		private readonly PositionedObjectsCollection space;		

		public MainForm()
		{
			InitializeComponent();

			space = GlobalObjects.Inst.Get<PositionedObjectsCollection>("Space");
		    objectsDrawer = new ObjectsCollectionDrawer(); 
			
			coordinateSurface = new PositionedObject();
            space.Add(coordinateSurface);
            objectsDrawer.Drawers.Set(coordinateSurface, new CoordinateSurfaceDrawer());

			cursorCoordinate = new PositionedObject();
			space.Add(cursorCoordinate);
            objectsDrawer.Drawers.Set(cursorCoordinate, new CoordinateDrawer());

			canvas.Viewport = new Viewport(new PointF(0, 0), canvas.Size);
			canvas.Objects = space;
		    canvas.ObjectsDrawer = objectsDrawer;
			canvas.CursorCoordinate = cursorCoordinate;

		    drawCoordinateSurfaceCheckBox.Checked = space.Contains(coordinateSurface);
			drawMouseCoordinateCheckBox.Checked = space.Contains(cursorCoordinate);

            linearFractalDrawer = new LinearFractalDrawer { FractalPen = new Pen(Color.FromArgb(0, 255, 0)), FractalDeep = 2 };
            drawBaseLinesCheckBox.Checked = linearFractalDrawer.DoesDrawBaseLines;
            drawDeepestOnlyCheckBox.Checked = linearFractalDrawer.DoesDrawDeepestFragmentsOnly;
            fractalDeepSlider.Value = linearFractalDrawer.FractalDeep;

            var fractals = new List<string>(LinearFractalLibrary.Keys);
            fractals.Sort();
            fractalsComboBox.Items.AddRange(fractals.ToArray());
		    fractalsComboBox.SelectedItem = "Piphagor tree";
		}

        private void SetupFractalObject()
        {
            if (linearFractal != null)
            {
                if (space.Contains(linearFractal))
                    space.Remove(linearFractal);
                if (objectsDrawer.Drawers.Contains(linearFractal))
                    objectsDrawer.Drawers.Remove(linearFractal); 
            }

            linearFractal = LinearFractalLibrary.Get((string)fractalsComboBox.SelectedItem);
            linearFractal.Scale(100);
            linearFractal.Rotate((float)Math.PI);

            space.Add(linearFractal);
            objectsDrawer.Drawers.Set(linearFractal, linearFractalDrawer);
            canvas.Fractal = linearFractal;

            fractalDeepSlider.Value = 2;
        }
		
		private void fractalDeepSlider_ValueChanged(object sender, EventArgs e)
		{
			linearFractalDrawer.FractalDeep = fractalDeepSlider.Value;
			DrawObjects();
		}		

		private void drawBaseLinesCheckBox_Click(object sender, EventArgs e)
		{
			linearFractalDrawer.DoesDrawBaseLines = drawBaseLinesCheckBox.Checked;
			DrawObjects();
		}

		private void DrawObjects()
		{
			canvas.DrawObjects();
		}

		private void drawDeepestOnlyCheckBox_Click(object sender, EventArgs e)
		{
			linearFractalDrawer.DoesDrawDeepestFragmentsOnly = drawDeepestOnlyCheckBox.Checked;
			DrawObjects();
		}

		private void drawCoordinateSurfaceCheckBox_Click(object sender, EventArgs e)
		{
		    objectsDrawer.Drawers[coordinateSurface].IsVisible = drawCoordinateSurfaceCheckBox.Checked;
			DrawObjects();
		}		

		private void drawMouseCoordinateCheckBox_Click(object sender, EventArgs e)
		{
            objectsDrawer.Drawers[cursorCoordinate].IsVisible = drawMouseCoordinateCheckBox.Checked;
			DrawObjects();
		}

        private void fractalColorButton_Click(object sender, EventArgs e)
        {
            colorDialog.Color = linearFractalDrawer.FractalPen.Color;
            colorDialog.ShowDialog();
            linearFractalDrawer.FractalPen.Color = colorDialog.Color;
            DrawObjects();
        }

        private void fractalsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            SetupFractalObject();
            DrawObjects(); 
        }
	}
}
